@@ -9,6 +9,9 @@
 #define _NGX_CORE_H_INCLUDED_
 
 
+#include <ngx_config.h>
+
+
 typedef struct ngx_module_s      ngx_module_t;
 typedef struct ngx_conf_s        ngx_conf_t;
 typedef struct ngx_cycle_s       ngx_cycle_t;
@@ -68,6 +71,7 @@ typedef void (*ngx_connection_handler_pt)(ngx_connection_t *c);
 #endif
 #include <ngx_radix_tree.h>
 #include <ngx_times.h>
+#include <ngx_rwlock.h>
 #include <ngx_shmtx.h>
 #include <ngx_slab.h>
 #include <ngx_inet.h>
